##parameters=date, with_time=0, ignore_unset_time=0

format = context.vs_calendarwidget_formats()['python_fmt']

try:
    if with_time and not (ignore_unset_time and date.TimeMinutes() == '00:00'):
        format += ' %H:%Mh' 
    return date.strftime(format)
except:
    return ''



################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther
################################################################

from zope import interface

class IVSEvent(interface.Interface):
    """ marker interface for VSEvent """

class IVSSubEvent(IVSEvent):
    """ marker interface for VSSubEvent """




from cStringIO import StringIO
from dateutil.rrule import YEARLY, MONTHLY, WEEKLY

from AccessControl import Unauthorized
from DateTime.DateTime import DateTime
from Products.ATContentTypes.lib.calendarsupport import rfc2445dt, vformat, foldLine, ICS_EVENT_END, ICS_EVENT_START
from Products.ATContentTypes.lib.calendarsupport import VCS_EVENT_START, VCS_EVENT_END
from Products.CMFCore.utils import getToolByName

from dateable.kalends import IRecurringEvent


def date_for_display(event):
    """ Return dict containing pre-calculated information for 
        building a <start>-<end> date string. Keys are
       'from_str' - date string for start date
       'to_str' - date string for end date
       'same_day' - event ends on the same day
    """

    toLocalizedTime = event.toLocalizedTime
    from_str = toLocalizedTime(event.start(), long_format=False)
    to_str = toLocalizedTime(event.end(), long_format=False)
    same_day = from_str==to_str

    if not event.getWholeDay():
        from_str = toLocalizedTime(event.start(), long_format=True)
        to_str = toLocalizedTime(event.end(), long_format=True)
    if from_str == to_str:
        result = dict(from_str=from_str, to_str=None, same_day=same_day)
    else:
        result = dict(from_str=from_str, to_str=to_str, same_day=same_day)
    if not event.getUseEndDate() and result.has_key('to_str'):
        del result['to_str']
    return result


def _dateForWholeDay(dt):
    """ Replacement for rfc2445dt() for events lasting whole day in 
        order to get the date string according to the current time zone.
        rfc2445dt() returns the date string according to UTC which is
        *not* what we want!
    """
    return dt.strftime('%Y%m%d')


def _dateStringsForEvent(event):
    # Smarter handling for all-day events
    data_dict = date_for_display(event)
    if event.getWholeDay():
        # For all-day events we must not include the time within
        # the date-time string
        start_str = _dateForWholeDay(event.start())[:8]
        if data_dict['same_day']:
            # one-day events end with the timestamp of the next day
            # (which is the start data plus 1 day)
            end_str = _dateForWholeDay(event.start() + 1)[:8]
        else:
            # all-day events lasting several days end at the next
            # day after the end date
            end_str = _dateForWholeDay(event.end() + 1)[:8]
    else:
        # default (as used in Plone)
        start_str = rfc2445dt(event.start())
        end_str = rfc2445dt(event.end())

    return start_str, end_str


def getICal(event):
    start_str, end_str = _dateStringsForEvent(event)
    out = StringIO()
    map = {
        'dtstamp'   : rfc2445dt(DateTime()),
        'created'   : rfc2445dt(DateTime(event.CreationDate())),
        'uid'       : event.UID(),
        'modified'  : rfc2445dt(DateTime(event.ModificationDate())),
        'summary'   : vformat(event.Title()),
        'startdate' : start_str,
        'enddate'   : end_str,
        }
    out.write(ICS_EVENT_START % map)

    description = event.Description()
    out.write(foldLine('DESCRIPTION:%s\n' % vformat(description)))

    location = event.getLocation()
    if location:
        out.write('LOCATION:%s\n' % vformat(location))

    try:
        atts = event.getAttachments()
    except (KeyError,AttributeError):
        atts = []
    res = []
    mtool = getToolByName(event, 'portal_membership')
    for d in range(len(atts)):
        try:
            obj = atts[d]
        except Unauthorized:
            continue
        if obj not in res:
            if mtool.checkPermission('View', obj):
                res.append(obj)
    if res:
        for r in res:
            out.write('ATTACH;VALUE=URI:%s\n' % r.absolute_url())

    eventType = event.getEventType()
    if eventType:
        out.write('CATEGORIES:%s\n' % ','.join(eventType))

    # TODO  -- NO! see the RFC; ORGANIZER field is not to be used for non-group-scheduled entities
    # ORGANIZER;CN=%(name):MAILTO=%(email)

    # Attendees
    for att in event.getAttendees():
        if att['show']:
            out.write('ATTENDEE;CN="%s":invalid:nomail\n' % vformat(att['name']))

    # Recurrency
#    if IRecurringEvent.providedBy(event):
    # ATT: better check interface?
    if hasattr(event, 'frequency'):    
        d_map = ['MO','TU','WE','TH','FR','SA','SU']
        freq = event.getFrequency()
        if freq != -1:
            if freq == YEARLY:
                f_str = 'YEARLY'
            elif freq == MONTHLY:
                f_str = 'MONTHLY'
            elif freq == WEEKLY:
                f_str = 'WEEKLY'
            else: 
                f_str = 'DAILY'
            days = event.getWeekdays()
            bysetpos = event.getBysetpos()
            if days and bysetpos: 
                    d_str = ",".join([d_map[int(i)] for i in days])
                    rrule_str = 'RRULE:FREQ=%s;INTERVAL=%d;BYSETPOS=%s;BYDAY=%s' % (f_str, event.interval,bysetpos, d_str )
            else: 
                rrule_str = 'RRULE:FREQ=%s;INTERVAL=%d' % (f_str, event.interval)

            if event.until:
                rrule_str = "%s;UNTIL=%s" %(rrule_str, rfc2445dt(event.until))

            out.write("%s\n" % rrule_str) 

    # Contact information
    cn = []
    contact = event.contact_name()
    if contact:
        cn.append(contact)
    phone = event.contact_phone()
    if phone:
        cn.append(phone)
    email = event.contact_email()
    if email:
        cn.append(email)
    if cn:
        out.write('CONTACT:%s\n' % vformat(', '.join(cn)))

    url = event.event_url()
    if url:
        out.write('URL:%s\n' % url)

    # allow derived event types to inject additional data for iCal
    try:
        event.getICalSupplementary(out)
    except AttributeError:
        pass

    out.write(ICS_EVENT_END)
    return out.getvalue()


def getVCal(event):
    """get vCal data """

    start_str, end_str = _dateStringsForEvent(event)
    out = StringIO()
    map = {
        'dtstamp'   : rfc2445dt(DateTime()),
        'created'   : rfc2445dt(DateTime(event.CreationDate())),
        'uid'       : event.UID(),
        'modified'  : rfc2445dt(DateTime(event.ModificationDate())),
        'summary'   : vformat(event.Title()),
        'startdate' : start_str,
        'enddate'   : end_str,
        }
    out.write(VCS_EVENT_START % map)
    description = event.Description()
    if description:
        out.write(foldLine('DESCRIPTION:%s\n' % vformat(description)))
    location = event.getLocation()
    if location:
        out.write('LOCATION:%s\n' % vformat(location))

    # allow derived event types to inject additional data for iCal
    try:
        event.getVCalSupplementary(out)
    except AttributeError:
        pass

    out.write(VCS_EVENT_END)
    return out.getvalue()



################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther 
################################################################

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Globals import InitializeClass

class iCalendarView(BrowserView):
    """ ICal export"""

    def icalendar_export(self, events=[], **kw):
        """ iCal export for multiple events """

        if not events:
            catalog = getToolByName(self.context, 'portal_catalog')
            calendar = getToolByName(self.context, 'portal_calendar')

            query = kw.copy()
            query['path'] = '/'.join(self.context.getPhysicalPath())
            cal_types = list(calendar.getCalendarTypes())
            query.update(dict(portal_type=cal_types))

            brains = catalog(query)
            events = [b.getObject() for b in brains]

        try:
            relcalid = self.context.UID()
        except AttributeError:
            relcalid = self.context.portal_url.getPortalObject().getId()

        result = list()
        write = result.append
        write('BEGIN:VCALENDAR')
        write('VERSION:2.0')
        write('X-WR-CALNAME:%s' % self.context.Title().upper())
        write('PRODID:-//Plone 3-vs.event\, Inc//(C) ZOPYX Ltd & Co. KG//EN')
        write('X-WR-RELCALID:%s' % relcalid)
        write('X-WR-TIMEZONE:Europe/Berlin')
        write('CALSCALE:GREGORIAN')
        write('METHOD:PUBLISH')

        for event in events:
            for line in event.getICal().split('\n'):
                write(line)

        write('END:VCALENDAR')

        body = '\n'.join(result)
        setHeader = self.context.request.response.setHeader
        setHeader('Content-Length', len(body))
        setHeader('Content-Type', 'text/x-vcalendar')
        setHeader('Content-Disposition', 'attachment; filename=plone-cal.ics')
        return self.context.request.response.write(body)

    def icalendar_export_event(self):
        """ iCal export for a single event (including subevents) """

        events = [self.context]
        try:
            events.extend(self.context.getSubEvents())
        except (KeyError, AttributeError):
            pass

        return self.icalendar_export(events=events)

InitializeClass(iCalendarView)



# -*- coding: iso-8859-15 -*-

from copy import deepcopy

""" Some helper methods for schema manipulation """

def invisibleFields(schema, *fields):
    """ makes a list of fields invisible """
    for name in fields:
        schema[name].widget.visible = False


def removeFields(schema, *fields):
    """ remove fields from a schema """
    for name in fields:
        del schema[name]


def removeActions(actions, *action_ids):
    """ Create a copy of the action datastructure to avoid overwriting
        of existing action definitions and mark some actions as invisible.
    """

    actions = deepcopy(actions)
    for action in actions:
        if action['id'] in action_ids:
            action['visible'] = 0
    return actions


def changeActionURL(actions, action_id, url):
    """changed the URL for an action without redefining it """
    for action in actions:
        if action['id'] == action_id:
            action['action'] = url


def restrictTypes(folder, types):
    """ set type restrictions to the folder object """

    folder.setConstrainTypesMode(1)
    folder.setImmediatelyAddableTypes(types)
    folder.setLocallyAllowedTypes(types)


def safeCreate(folder, id, portal_type):
    """ return folder[id], create it it does not exisit """
    if not id in folder.objectIds():
        folder.invokeFactory(portal_type, id)
    return getattr(folder, id)



################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther
################################################################

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Globals import InitializeClass
from vs.event.config import *
from vs.event.content import event_util
import random

class VSView(BrowserView):
    """ """

    def add_supplementary_event(self):
        """ create a new supplementary event"""
        calendar = getToolByName(self.context, 'portal_calendar')
        if not calendar.vs_event_supplementary_events:
            raise RuntimeError('portal_calendar.vs_event_supplementary_events '
                               ' is not enabled')
        id = self.context.getId() + '-' + str(random.random())
        self.context.invokeFactory('VSSubEvent', id=id)
        event = self.context[id]
        event.setTitle(self.context.Title())
        event.setDescription(self.context.Description())
        event.setExcludeFromNav(True)
        sub_events = self.context.getSubEvents() or []
        sub_events.append(event)
        self.context.setSubEvents(sub_events)
        self.request.response.redirect(event.absolute_url() + '/edit')

    def date_for_display(self):
        """ Return dict containing pre-calculated information for 
            building a <start>-<end> date string.
        """
        return event_util.date_for_display(self.context)

InitializeClass(VSView)
